#ifndef DEBTAGS_PACKAGES_H
#define DEBTAGS_PACKAGES_H

/*
 * Central class to perform package searches
 *
 * Copyright (C) 2003  Enrico Zini <enrico@debian.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#pragma interface

#include "DebtagsConsumer.h"
#include "DebtagsFilter.h"
#include "PackageDB.h"
#include "DebtagsVocabulary.h"

#include <InputMerger.h>

#include <map>
#include <vector>

namespace Debtags
{

class Packages
{
public:
	class Maintainer
	{
	protected:
		std::string _name;
		std::vector<Package*> _packages;

		// Create a maintainer with no packages
		Maintainer(std::string name) throw ()
			: _name(name) {}
		// Create a maintainer with the given package
		Maintainer(std::string name, Package* pkg) throw ()
			: _name(name)
		{
			_packages.push_back(pkg);
		}
		// Add a package to the maintainer
		void addPkg(Package* pkg) throw () { _packages.push_back(pkg); }

	public:
		const std::string& name() const throw () { return _name; }
		const std::vector<Package*>& packages() const throw () { return _packages; }

		friend class AvailableReader;
	};
protected:
	typedef std::map<std::string, Package*> pool_t;
	mutable pool_t pool;
	mutable InputMerger<Package*> _tagdb;
	std::vector<Maintainer*> _maintainers;
	mutable PackageDB _packagedb;
	mutable Vocabulary _vocab;

	bool availableLoaded;
	mutable bool debtagsLoaded;
	
	Package* obtain(const std::string& name) const throw ();
	void loadAvailable() throw ();
	void loadDebtags() const throw ();
	void sendIfMatch(Package* pkg, PackageConsumer& cons, Filter& filter) throw ();

public:
	Packages() throw ();
	~Packages() throw ();

	// Apply a tag change to the package DB
	TagcollChange<Package*, std::string> applyChange(const TagcollChange<Package*, std::string>& change) throw ();

	// Get a list of all defined Debian Package Tags
	OpSet<std::string> getAllTags() const throw ();

	// Get the set of all tags in this collection that appear in tagsets
	// containing `ts'
	OpSet<std::string> getCompanionTags(const OpSet<std::string>& ts) const throw ();

	// Get the tagset for package `pkg'
	OpSet<std::string> getTagsetForItem(Package * const pkg) const throw ();

	// Get the Package* structure given the package name
	Package* find(const std::string& name) throw ();

	// Output the whole package list to `cons'
	void output(PackageConsumer& cons) throw ();

	// Output the filtered package list to `cons'
	void output(PackageConsumer& cons, Filter& filter) throw ();

	// Output list of packages related to `pivot' to `cons'
	void outputRelated(PackageConsumer& cons, Package* pivot, int distance = 1) throw ();

	// Save tag changes to the persistent state
	void save() throw (FileException, ParserException, SystemException);

	// Send changes via mail to central archive
	void send() throw (FileException, ParserException, SystemException, ConsistencyCheckException);
	
	friend class Package;
	friend class AvailableReader;
	friend class ItemsToPackages;
};

};

// vim:set ts=4 sw=4:
#endif
